c initialize_nodal_geometry

      subroutine initialize_nodal_geometry
     & (xdouar,ydouar,hdouar,nnodedouar,
     &  nnodemax,nnode,x,y,h,fix,delta,surfscale,
     &  run_name,nrun_name,
     &  ifixx0,ifixx1,ifixy0,ifixy1)

c subroutine to read in initial nodal information

c INPUT: nnodemax      = maximum number of nodes that can be read
c        run_name      = name of the subdirectory where the run output
c                        will be stored
c        nrun_name     = length of the character string run_name

c OUTPUT: nnode        = number of nodes read
c         x(nnode)     = x-location of nodes (in m)
c         y(nnode)     = y-location of nodes (in m)
c         h(nnode)     = height of nodes (in m)
c         fix(nnode)   = boundary condition (=0 means node height is fixed;
c                        =1 means node height is free)
c         delta        = mean nodal spacing (in m)
c         surfscale    = mean nodal surface (a nodal surface is the surface
c                        of the part of the landscape associated with a node)
c                        (in m**2)

c subroutines called:
c - debug
c - random
c - iread_but_skip_comment
c - read_but_skip_comment

      common /vocal/ ivocal

      double precision xdouar(nnodedouar),ydouar(nnodedouar)
      double precision hdouar(nnodedouar)
      integer ifixx0,ifixx1,ifixy0,ifixy1

      real          x(*),y(*),h(*)
      real          fix(*)
      character     run_name*256

      iread=0

      if (iread.eq.0) then

      nnode=nnodedouar
        if (nnode.gt.nnodemax) then
        stop 'Too many nodes...'
        endif

      sidex=maxval(xdouar)
      sidey=maxval(ydouar)

      surfscale=sidex*sidey/nnode
      delta=sqrt(surfscale)

c this is a bit of random noise put on the initial grid
c note that the noise is not used for the nodes along the boundary

!      if (ivocal.eq.1) call debug ('random$',0)
!      call random (x,nnode)
!      call random (y,nnode)
!      call random (h,nnode)
!      if (ivocal.eq.1) call debug ('initialize_nodal_geometry$',1)
 
        do inode=1,nnode
        x(inode)=xdouar(inode)
        y(inode)=ydouar(inode)
        h(inode)=hdouar(inode)
        fix(inode)=1.
        enddo

      xmin=minval(xdouar)
      xmax=maxval(xdouar)
      ymin=minval(ydouar)
      ymax=maxval(ydouar)
      dx=(xmax-xmin)/1.e5
      dy=(ymax-ymin)/1.e5
        do inode=1,nnode
        if (x(inode).lt.xmin+dx.and.ifixx0.eq.1) fix(inode)=0.
        if (x(inode).gt.xmax-dx.and.ifixx1.eq.1) fix(inode)=0.
        if (y(inode).lt.ymin+dy.and.ifixy0.eq.1) fix(inode)=0.
        if (y(inode).gt.ymax-dy.and.ifixy1.eq.1) fix(inode)=0.
        enddo

      else

      open (7,file=run_name(1:nrun_name)//
     &      '/cascade.node.in',status='old')

      call iread_but_skip_comment (7,1,nnode)
        if (nnode.gt.nnodemax) then
        stop 'Too many nodes...'
        endif

      call read_but_skip_comment (7,nnode,x)
      call read_but_skip_comment (7,nnode,y)
      call read_but_skip_comment (7,nnode,h)
      call read_but_skip_comment (7,nnode,fix)
      xmin=x(1)
      xmax=x(1)
      ymin=y(1)
      ymax=y(1)
        do i=1,nnode
        xmin=min(xmin,x(i))
        xmax=max(xmax,x(i))
        ymin=min(ymin,y(i))
        ymax=max(ymax,y(i))
        enddo
      sidex=xmax-xmin
      sidey=ymax-ymin
      snode=sqrt(float(nnode))
      delta=(sidex/snode+sidey/snode)/2.
      surfscale=sidex*sidey/nnode

      close (7)

      endif

        if (nnode.lt.3) then
        stop 'nnode too small...'
        endif

        if (delta.le.0.) then
        stop 'delta must be greater than 0...'
        endif

        if (surfscale.le.0.) then
        stop 'surfscale must be greater than 0...'
        endif

        if (sidex.le.0.) then
        stop 'sidex must be greater than 0...'
        endif

        if (sidey.le.0.) then
        stop 'sidey must be greater than 0...'
        endif

      return
      end
